/************************************************************
*                                                           *
* Politecnico di Milano - University of Illinois at Chicago *
*                                                           *
*    Networked Operating Systems exam - Professor Piuri     *
*                                                           *
* Assignment by Daniele Paolo Scarpazza                     *
* Copyright (C) 2000, by Daniele Paolo Scarpazza.           *
*               All rights reserved.                        *
*                                                           *
************************************************************/

/*
  Note to code maintainers:

  WARNING: this program listing contains embedded documentation.
  Please respect writing conventions, otherwise modifications could
  result in inconsistent or partially missing generated documentation. 

  Daniele Paolo Scarpazza

*/

#define  PORT              12345
#define  ID_BUF_SIZE         255
#define  TXT_BUF_SIZE      65535
#define  ALLOC_GRANULARITY    10

#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/unistd.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <assert.h>
#include <fcntl.h>
#include <time.h>
#include "termgraph.h"

//////////////////////////////////////////////////////////////////////////////////////////////////////////

/** State identifiers. 
  * @name Mailbox State identifiers 
  * These are the identifiers used by the finite state machine implemented by the mailbox.
  */
//@{
/// State identifier: the mailbox has not yet been initialised
#define STATE_UNNAMED        0
/// State identifier: the mailbox is reading its name
#define STATE_EXTR_NAME      1
/// State identifier: the mailbox is idle
#define STATE_WAITING        2
/// State identifier: the mailbox is receiving the destination for an incoming message
#define STATE_EXTR_DEST      3
/// State identifier: the mailbox is receiving the low-order byte of the length word for an incoming message
#define STATE_EXTR_LEN1      4
/// State identifier: the mailbox is receiving the high-order byte of the length word for an incoming message
#define STATE_EXTR_LEN2      5
/// State identifier: the mailbox is receiving the body of an incoming message 
#define STATE_EXTR_BODY      6
/// State identifier: the mailbox is acknowledging the sender about the completion of the message delivery
#define STATE_ACKNOWLEDGING  7
/// State identifier: the mailbox is sending a message to the attached socket
#define STATE_SENDING        8
//@}

/** Mailbox server class implementation.
  *
  */
 
class CMailbox
{
 protected:
  /// Use flag. Indicates whether the mailbox is in use or not.
  bool    m_bIsUsed;
  /// Malbox Name. Contains the name of the mailbox.
  char    m_strName[ID_BUF_SIZE+1];
  /// Socket file descriptor number. This is the file descriptor number of the socket associated to the mailbox.
  int     m_iSocket; 
  /// Mailbox state. This is the current state of the finite state machine which implements the mailbox.
  int     m_iState;
  /// Current input buffer position. This is the current position in the input buffer of the mailbox, it is u
  int     m_iPosition;
  /// Input message length. This is the read lenght of the body of the message which is being read.
  int     m_iMsgLen;
  /// Output buffer size. This is the size of the allocated output data buffer
  int     m_iOutBufferSize;
  /// Used portion of the output data buffer
  int     m_iOutBufferLen;
  /// Recipient mailbox identifier.
  char    m_strDest[ID_BUF_SIZE+1];
  /// Text of the incoming message
  char    m_strBody[TXT_BUF_SIZE+1];
  /// Dynamically allocated pending output data buffer
  char  * m_strOutBuffer;

 public:
  
  /// Default constructor. Initializes data to empty values.

  CMailbox()                            
  {
    m_bIsUsed        = false;
    m_strName[0]     = 0;
    m_iState         = STATE_UNNAMED;
    m_iOutBufferLen  = 0;
    m_iOutBufferSize = 0;
    m_strOutBuffer   = 0; 
  };
  
  /// Initializes a new mailbox. Associates an unused mailbox to a socket.
  Setup(int iSocket)     
  { 
   m_bIsUsed         = true; 
   m_iSocket         = iSocket;  
   m_strName[0]      = 0;
   m_iState          = STATE_UNNAMED;
   m_iOutBufferLen   = 0; 
   m_iOutBufferSize  = 0; 
 
   printf("Mailbox for socket %i has just been created.",m_iSocket); OK;
  }

  /// Default destructor. 
  ~CMailbox()                           { Close(); };

  /** Close the mailbox. 
      Resets the mailbox associated data and indicates that the mailbox is unused 
      and can be set up for new mailboxes. */  
  Close()                               
  { 
    printf(" %i:\"%s\" mailbox has just left server.",m_iSocket,m_strName); WAIT;
    m_strName[0]=0;
    m_bIsUsed=false;
    m_iSocket=0;
    if (m_iOutBufferSize) 
      {
	free(m_strOutBuffer);
        m_strOutBuffer = 0;
	m_iOutBufferSize = 0;
      }
    OK;
  }

  /** Returns true if the mailbox is in use.
    * The mailbox is defined to be in use when it has an associated socket and a remote process
    * connected to it, even if the remote process has not yet indicatet its name.
    */ 
  bool IsUsed()                         { return m_bIsUsed; }
  
  /** Returns the mailbox name.
    * Returns the name of the mailbox which has been chosen by the remote process connected to it.
    */
  const char * GetName()                { return m_strName; }

  /** Returns the associated socket.
    * Returns the file descriptor of the socket currently associated to the mailbox. 
    */
  const int    GetSocket()              { return m_iSocket; }
  
  /** Returns the message recipient.
    * Returns the destination mailbox of the incoming message, if any.
    * When the mailbox is not receiving any message (i.e. it is in the STATE\_WAITING state), this
    * is the destination of the last incoming message.
    */  
  const char * GetMsgDest()             { return m_strDest; }
  
  /** Returns the message body length.
    * Returns the length of the body (therefore not counting recipient mailbox identifier, string terminator,
    * length bytes) of the incoming message, if any. When the mailbox is not receiving any message 
    * (i.e. it is in the STATE_WAITING state), this is the body length of the last incoming message.
    */
  const int    GetMsgLen()              { return m_iMsgLen; }
  
  /// Returns the incoming message body text
  const char * GetMsgBody()             { return m_strBody; }
  
  /** Indicates the delivery success.
    * Indicates to the mailbox that the last incoming message has been (or not) correctly delivered
    * to destination mailbox. This method notifies the success (or insuccess) to the process owning the mailbox.
    */ 
  void Delivered(bool bSuccess)
  {
    if (m_iState!=STATE_ACKNOWLEDGING)
      {
	printf(" %i:\"%s\": Erroneous call to Delivered();\n",m_iSocket, m_strName);
	return;
      }
    if (bSuccess)
      {
	printf(" %i:\"%s\": Sending positive acknowledgment to sender.",m_iSocket, m_strName);
	EnqueueData("\1",1);
	OK;
      }    
    else
      {
	printf(" %i:\"%s\": Sending negative acknowledgment to sender.",m_iSocket, m_strName);
	EnqueueData("\0",1);
        OK;
      }    
    m_iState=STATE_WAITING;
  } 

  /** Returns true if the socket has pending output.
    * Returns true when the output pending data buffer is not empty.
    * This represent a sort of request for the caller to call the WriteEnqueuedData() member
    * as soon as the associated socket is ready for writing.
    */ 
  bool HasPendingOutput()
  {
    return m_iOutBufferLen>0;
  }
  
  /** Enqueue output data.
    * Appends outgoing data to the the mailbox output buffer.
    */
  void EnqueueData(const char * strData, int iLen)
  {
    if (m_iState!=STATE_WAITING && m_iState!=STATE_ACKNOWLEDGING)
      {
        printf(" %i:\"%s\" Cannot enqueue data, inconsistent state.\n",m_iSocket, m_strName);
       	return;
      } 
  
    if (iLen<=0) return;
    
    if (m_iOutBufferLen + iLen >= m_iOutBufferSize)
      {   
        m_iOutBufferSize += iLen * ALLOC_GRANULARITY;
        printf(" %i:\"%s\" Reallocating outgoing data buffer, new length=%i;\n",m_iSocket, m_strName, m_iOutBufferSize);
   	m_strOutBuffer = realloc(m_strOutBuffer, m_iOutBufferSize);
      }
#ifdef NDEBUG
    printf(" %i:\"%s\" Enqueuing data, length=%i;\n",m_iSocket, m_strName, iLen);
#endif
    memcpy(m_strOutBuffer + m_iOutBufferLen, strData, iLen);
    m_iOutBufferLen+=iLen;        
  }

  /** Write outgoing data.
    * Tries to write to the socket the outgoing pending data, through a non-blocking write() call.
    */
  bool WriteEnqueuedData()
  {
    int rc;
    printf(" %i:\"%s\": Writing Enqueued data, length=%i;\n",m_iSocket, m_strName, m_iOutBufferLen);
    fcntl(m_iSocket,F_SETFL,O_NONBLOCK);
   
    rc=write(m_iSocket, m_strOutBuffer, m_iOutBufferLen);
#ifdef NDEBUG    
    printf(" %i:\"%s\": Written %i bytes\n",m_iSocket, m_strName, rc);
#endif
    memmove(m_strOutBuffer, m_strOutBuffer+rc, rc);  
    m_iOutBufferLen-=rc;
  } 

  /** Notifies mailbox of a character arrival.
    * Updates the mailbox finite state machine with a new input
    */
  bool CharReceived(unsigned char ch)
  {
    // returns true when it has a ready message to deliver
    
#ifdef NDEBUG
    printf(" %i:\"%s\"<-'%c';\n",m_iSocket, m_strName, ch);
#endif    

    switch(m_iState)
      {
      case STATE_UNNAMED:
        m_iState = STATE_EXTR_NAME;
        m_iPosition   = 0;
        //ATTN: no break ! Prosecutes reading first chr
     
      case STATE_EXTR_NAME:
	m_strName[m_iPosition]=ch;
        m_iPosition++;
	if (!ch)
	  {
	    m_iState  = STATE_WAITING;
            printf(" %i:\"%s\": Mailbox has just been created !\n",m_iSocket, m_strName);
            // TODO: check name duplication
            
            // Send acknowledgment
            write(m_iSocket,"\1",1);
	  }
	break;
	
      case STATE_WAITING:
	m_iState      = STATE_EXTR_DEST;
        m_iPosition   = 0;
        //ATTN: no break ! Prosecutes reading first chr
      
      case STATE_EXTR_DEST:
	m_strDest[m_iPosition]=ch;
        m_iPosition++;
	if (!ch) m_iState  = STATE_EXTR_LEN1;
	break;
	
      case STATE_EXTR_LEN1:
        // read the high-order byte of the length word
        m_iMsgLen=ch * 256;
      	m_iState = STATE_EXTR_LEN2;
        break;
	
      case STATE_EXTR_LEN2:
        // read the low-order byte
	m_iMsgLen+=ch;
        printf(" %i:\"%s\": Message for \"%s\", %i bytes long ;\n",m_iSocket, m_strName, m_strDest, m_iMsgLen);
      	m_iState = STATE_EXTR_BODY;
        m_iPosition = 0;
	
        if (!m_iMsgLen)
	  {
	    //zero-length message
	    m_iState=STATE_ACKNOWLEDGING;
	    m_strBody[0]=0;
	    break;
	  }
        break;

      case STATE_EXTR_BODY: 
	m_strBody[m_iPosition]=ch;
	m_iPosition++;
	m_strBody[m_iPosition]=0;
        if (m_iPosition==m_iMsgLen)
	  { 
	   printf(" %i:\"%s\": Message text is \"%s\"\n",m_iSocket, m_strName, m_strBody);
	   m_iState=STATE_ACKNOWLEDGING;
	   // Send acknowledgment
	   return true; 
           break; //fake
	  } 
	break;
      }
    return false;
  }
 
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////

/** Mailbox collection class.
 *
 */

class CMailboxCollection					
{								
protected:		
  /** Internal mailbox array.
    * This data member points to a dynamically allocated area which contains the pointers to all the 
    * allocated mailbox objects.
    */					
  CMailbox ** m_ppmbx;		
				
  /** Number of allocated mailboxes.
    * This is the number of dynamically allocated mailbox objects. 
    */
  int m_iAllocated;

  /** Number of used mailboxes.
    * This is the count of the mailboxes currently in use, that is to say the mailboxes which have
    * a name and an associated process and socket.
    */
  int m_iUsed;			
 								
public:				
  /// Default constructor. Initializes the mailbox collection with empty data.			       
  CMailboxCollection() {m_ppmbx=NULL; m_iAllocated=m_iUsed=0; }	
					
  /// Default destructor. Deletes all the contained mailbox objects and frees the used memory.
  ~CMailboxCollection() 					
  { 								
    for (int i=0; i++; i<m_iAllocated)  		
	delete m_ppmbx[i];	   
    free(m_ppmbx);
  }								

  /** Finds a mailbox by name. 
    * Returns  the number of the first mailbox which has the given name, starting from the given point.
    * @param   strName this is the name of the mailbox you want to find.
    * @param   iStartPos the number to start from in the search.
    * @return  the number of the mailbox which has the given name, or the value -1 if 
    *          no mailboxes with the given name can be found.
    */ 
  int FindMailbox(const char * strName, int iStartPos=0) 
  {
     for (int i=iStartPos;i<m_iAllocated;i++)
      if (m_ppmbx[i]->IsUsed() && strcmp(strName,m_ppmbx[i]->GetName())==0)
	return i;
     return -1; 
  } 
 
  /** Finds a mailbox by socket. 
    * Returns the number of the mailbox which is associated to the given socket.
    * @param  iSocket this is the socket associated to the mailbox you want to find.
    * @return  the number of the mailbox which has the given name, or the value -1 if 
    *          no mailboxes with the given name can be found.
    */

  int FindMailbox(int iSocket) // by socket
  {
     for (int i=0;i<m_iAllocated;i++)
      if (m_ppmbx[i]->IsUsed() && iSocket==m_ppmbx[i]->GetSocket())
	return i;
     return -1; 
  } 

  /** Updates the socket write set.
    * Updates the set of sockets which should be kept under observation to flush pending outgoing data.
    * To achieve this, every used mailbox in the collection is examined to see whether it has pending data.
    */ 
  void UpdateWriteSet(fd_set * write_fd_set)
  {
    FD_ZERO(write_fd_set);
    for (int i=0;i<m_iAllocated;i++)
      if (m_ppmbx[i]->IsUsed() && m_ppmbx[i]->HasPendingOutput())
	{
          printf(" %i:\"%s\": Detected pending output data on socket.",m_ppmbx[i]->GetSocket(),m_ppmbx[i]->GetName());
          OK;
	  FD_SET(m_ppmbx[i]->GetSocket(),write_fd_set);
	}
  }

  /** Adds a new mailbox.
    * Sets a new mailbox up with the given socket number.
    * If every allocated mailbox is used, a new set of empty mailbox (of cardinality ALLOC\_GRANULARITY) is allocated.
    */
  bool AddMailbox(int iSocket)
  {
     if (m_iAllocated==m_iUsed)
      {
        int i=0;
        m_ppmbx = realloc(m_ppmbx,sizeof(CMailbox*)*(m_iAllocated + ALLOC_GRANULARITY));
        for (i=0;i<ALLOC_GRANULARITY;i++)
	{
	  m_ppmbx[i+m_iAllocated]=new CMailbox;
          assert(m_ppmbx[i+m_iAllocated]);
	}
        m_iAllocated += ALLOC_GRANULARITY;
      }
    
    for (int i=0;i<m_iAllocated;i++)
      if (!m_ppmbx[i]->IsUsed())
	{ 
	  m_ppmbx[i]->Setup(iSocket); 
	  m_iUsed++;
          return true;
	}
       
    return false;
  }
 
  /** Writes the enqueued data for the given socket.
    * Identifies the socket in the collection which has the given socket and, using its WriteEnqueuedData method,
    * asks it to write (if possible) pending outgoing data.
    */  

  void WriteEnqueuedData(int iSocket)
  {
    int iPos;
    if ((iPos=FindMailbox(iSocket))==-1)
      {
	printf("Error: cannot find mailbox associated to socket %i while trying to write enqueued data.\n",iSocket);
	return;
      }
    m_ppmbx[iPos]->WriteEnqueuedData();
  }
  
  /** Closes the mailbox which has a given name.
    * The mailbox with the given name is reinitialized, every pending data is lost.
    */
  bool CloseMailbox(const char * strName)
  {
    int iPos;
    if ((iPos=FindMailbox(strName))!=-1)
      return false;
    m_ppmbx[iPos]->Close();
    return true;
  }
  

  /** Closes the mailbox which has a given name.
    * The mailbox with the given socket is reinitialized, every pending data is lost.
    */ 
  bool CloseMailbox(int iSocket)
  {
    int iPos;
    if ((iPos=FindMailbox(iSocket))==-1)
      return false;
    m_ppmbx[iPos]->Close();
    return true;
  }

  /** Closes all the mailboxes.
    */
  void CloseMailboxes()
  {
    for (int i=0; i<m_iAllocated;i++)
      if (m_ppmbx[i]->IsUsed()) m_ppmbx[i]->Close();
  }
  
  /// Notify character. Notify the mailbox associated to given socket of a character arrival. 
  bool CharFromSocket(int iSocket, unsigned char ch)
  {
    int iPos, iPosTarget, iCountTargets=0;
    if ((iPos=FindMailbox(iSocket))==-1)
      {
	printf("Error: cannot find mailbox associated to socket %i while trying to dispatch incoming data.\n",iSocket);
	return false;
      }
    if (m_ppmbx[iPos]->CharReceived(ch))
      {
	if (strlen(m_ppmbx[iPos]->GetMsgDest()))
	  {
	    //ready to deliver
            iPosTarget=0;
            while (true)
	      {
		iPosTarget=FindMailbox(m_ppmbx[iPos]->GetMsgDest(), iPosTarget);
		if (iPosTarget==-1) break;
		iCountTargets++;
		
		unsigned char chLen1 =  m_ppmbx[iPos]->GetMsgLen() / 256,
		              chLen2 =  m_ppmbx[iPos]->GetMsgLen() % 256;
		printf("Delivering message from \"%s\" to \"%s\"\n",m_ppmbx[iPos]->GetName(),m_ppmbx[iPos]->GetMsgDest());
#ifdef NDEBUG		
		printf("Writing to socket %i the sender's name \"%s\"\n",m_ppmbx[iPosTarget]->GetSocket(), m_ppmbx[iPos]->GetName());
#endif
		// terminator is included
		m_ppmbx[iPosTarget]->EnqueueData(m_ppmbx[iPos]->GetName(),strlen(m_ppmbx[iPos]->GetName())+1);
		
#ifdef NDEBUG                		
		printf("Writing the number of bytes = %i\n", m_ppmbx[iPos]->GetMsgLen());
#endif
		m_ppmbx[iPosTarget]->EnqueueData( (char*) &chLen1,1);
		m_ppmbx[iPosTarget]->EnqueueData( (char*) &chLen2,1);
#ifdef NDEBUG
		printf("Writing the message body \"%s\"\n",m_ppmbx[iPos]->GetMsgBody());
#endif
		m_ppmbx[iPosTarget]->EnqueueData(m_ppmbx[iPos]->GetMsgBody(),m_ppmbx[iPos]->GetMsgLen());
		iPosTarget++;
	      } 
	    if (iCountTargets==0)
	      {
		printf("Error: cannot deliver message to non-existing mailbox \"%s\"\n",m_ppmbx[iPos]->GetMsgDest());
		m_ppmbx[iPos]->Delivered(false);	    
	      } else m_ppmbx[iPos]->Delivered(true);
	  } else {
	    // empty destination means broadcast 
	    printf("Broadcasting message from \"%s\"",m_ppmbx[iPos]->GetName()); OK;
	    for (int iPosTarget=0; iPosTarget<m_iAllocated;iPosTarget++)
	      {
		if (m_ppmbx[iPosTarget]->IsUsed() && iPos!=iPosTarget)
		  {
		    unsigned char chLen1 =  m_ppmbx[iPos]->GetMsgLen() / 256,
		      chLen2 =  m_ppmbx[iPos]->GetMsgLen() % 256;
		    
#ifdef NDEBUG	   
		    printf("Writing to socket %i the sender's name \"%s\"\n",m_ppmbx[iPosTarget]->GetSocket(), m_ppmbx[iPos]->GetName());
#endif		
		    // terminator included
		    m_ppmbx[iPosTarget]->EnqueueData(m_ppmbx[iPos]->GetName(),strlen(m_ppmbx[iPos]->GetName())+1);
		    
#ifdef NDEBUG              		
		    printf("Writing the number of bytes = %i\n", m_ppmbx[iPos]->GetMsgLen());
#endif
		    m_ppmbx[iPosTarget]->EnqueueData( (char*) &chLen1,1);
		    m_ppmbx[iPosTarget]->EnqueueData( (char*) &chLen2,1);
#ifdef NDEBUG
		    printf("Writing the message body \"%s\"\n",m_ppmbx[iPos]->GetMsgBody());
#endif
		    m_ppmbx[iPosTarget]->EnqueueData(m_ppmbx[iPos]->GetMsgBody(),m_ppmbx[iPos]->GetMsgLen());
		    
		  }
	      }
	    m_ppmbx[iPos]->Delivered(true);
	  }
      }
    return true;
  }

};

/** Mailbox collection instance.
  */
CMailboxCollection    mbxcoll;


/** SIGINT signal catcher.
  * Intercepts SIGINT signals (^C), in event of it closes all sockets and terminates program. 
  */
  
void ctl_c_catch(int i)
{
  OK;
  printmsg("SIGINT signal (^C) detected: closing sockets and quitting"); 
  WAIT;
  mbxcoll.CloseMailboxes();
  OK;
  exit(EXIT_FAILURE);
}    




//////////////////////////////////////////////////////////////////////////////////////////////////////////



/** Mailboxd daemon main().
  *
  */

int main()
{
  int iSocket;
  struct sockaddr_in    sinAddress, sinClientAddr;
  struct hostent     *  heHostinfo;
  int i;
  fd_set active_fd_set, read_fd_set, write_fd_set;
  time_t                timeNow;
  char                  strTime[256];

  signal(2,&ctl_c_catch);

  printf("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n"
	 " mailboxd server with advanced features - version 1.0  \n"
	 "                          _/                        \n"	
	 "                         _/                         \n"
	 "                        _/                          \n"
	 "   _/_/_/_/ _/_/_/     _/_/_/_/_/   _/     _/       \n"
	 "  _/     _/     _/    _/       _/     _/_/          \n"
	 " _/     _/     _/    _/       _/     _/_/           \n"
	 "_/     _/     _/    _/_/_/_/_/    _/     _/         \n"
	 "\n"
	 ""
	 " Written by Daniele Paolo Scarpazza \n"
	 "\n");
 
  printmsg(MOVE_TO_COL_ZERO "Trying terminal ANSI SUPPORT (showing all messages)");         fflush(0); sleep(1);
  printmsg(MOVE_TO_COL_ZERO "Trying terminal ANSI SUPPORT (showing all messages - wait)");  WAIT;      sleep(1);
  printmsg(MOVE_TO_COL_ZERO "Trying terminal ANSI SUPPORT (showing all messages - fail)");  FAIL_NE;   sleep(1);
  printmsg(MOVE_TO_COL_ZERO "Trying terminal ANSI SUPPORT (showing all messages -  ok )");  OK;        sleep(1);


 
  printmsg("Starting mailboxd server"); OK;

  printmsg("Creating socket"); WAIT;
  iSocket = socket(PF_INET,SOCK_STREAM,0);
  if (iSocket==-1) FAIL else OK;

  memset(&sinAddress,0,sizeof(sinAddress));
  sinAddress.sin_family      = AF_INET;
  sinAddress.sin_port        = htons(PORT);
  sinAddress.sin_addr.s_addr = htonl(INADDR_ANY);
 
  printmsg("Binding to socket");  
  if (bind(iSocket,(struct sockaddr*)&sinAddress, sizeof(sinAddress))<0) FAIL else OK;
  
  printmsg("Listening onto socket");  
  if (listen(iSocket,1)<0) FAIL else OK;
 
  FD_ZERO(&active_fd_set);
  FD_ZERO(&write_fd_set);
  FD_SET(iSocket, &active_fd_set);
 
  while(1)
   {
    mbxcoll.UpdateWriteSet(&write_fd_set);
    time(&timeNow);
    strcpy(strTime,ctime(&timeNow));
    strTime[strlen(strTime)-1]=0;
    printf("%s: ", strTime);
    printmsg("Waiting for events"); WAIT;
    read_fd_set = active_fd_set;
    if (select(FD_SETSIZE, &read_fd_set, &write_fd_set, NULL, NULL)<0) FAIL else OK;
     
     
     for (i=0; i < FD_SETSIZE; i++)
       if (FD_ISSET(i, &read_fd_set))
	 {
	   if (i==iSocket)
	     { 
	       int iSize = sizeof(sinClientAddr), iNewSocket; 
	       printmsg("Accepting connections"); WAIT;
	       
	       iNewSocket=accept(iSocket,(struct sockaddr *)&sinClientAddr,&iSize);
	       if (iNewSocket<0) FAIL else OK;
	       printf("Connect from host %s, port %hd, given socket %i \n",
		      inet_ntoa(sinClientAddr.sin_addr),
		      ntohs(sinClientAddr.sin_port),i);
	       OK;
	       mbxcoll.AddMailbox(iNewSocket);
	       FD_SET(iNewSocket, &active_fd_set);
	     } else {
	       char buffer[256];
	       int  nBytes;
	       
	       printf("Reading data on existing socket %i",i); 
	       printmsg(""); WAIT;
	       nBytes = read(i,buffer,255);
	       if (nBytes<0) FAIL else OK;
	       
	       if (nBytes==0) 
		 {
		   printf("Socket number %i has been closed.\n",i);
		   mbxcoll.CloseMailbox(i);
		   OK;
		   FD_CLR(i, &active_fd_set);
		 } else
		   for (int j=0;j<nBytes;j++)
		   mbxcoll.CharFromSocket(i,buffer[j]);
	     }
	 }
     
     for (i=0; i < FD_SETSIZE; i++)
       if (FD_ISSET(i, &write_fd_set))
	 {
	   mbxcoll.WriteEnqueuedData(i);
	 }
   }
}










