/************************************************************
*                                                           *
* Politecnico di Milano - University of Illinois at Chicago *
*                                                           *
*    Networked Operating Systems exam - Professor Piuri     *
*                                                           *
* Assignment by Daniele Paolo Scarpazza                     *
* Copyright (C) 2000, by Daniele Paolo Scarpazza.           *
*               All rights reserved.                        *
*                                                           *
************************************************************/

/*
  Note to code maintainers:

  WARNING: this program listing contains embedded documentation.
  Please respect writing conventions, otherwise modifications could
  result in inconsistent or partially missing generated documentation. 

  Daniele Paolo Scarpazza

*/

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/unistd.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <stdio.h>
#include <netdb.h>
#include "termgraph.h"
#include "remotemailbox.h"

;
/** Standard class constructor.
  * Simply initializes member variables to null or default values.
  * It sets the verbosity to the current value of RMBX_DEFAULT_VERBOSITY,
  * as it can be found in remotemailbox.h 
  * @see RMBX_DEFAULT_VERBOSITY
  * @see ~CRemoteMailbox();
  */
CRemoteMailbox::CRemoteMailbox()
{
  m_iSocket=0;
  m_bConnected=false;
  m_bVerbose= RMBX_DEFAULT_VERBOSITY;
  m_strMailBoxName[0]=0;
};

/** Standard class destructor.
  * Simply closes the associated socket, to disconnect pending communications, if any.
  * @see Disconnect();
  */
CRemoteMailbox::~CRemoteMailbox()
{
  Disconnect();
};

/** Disconnects from the server.
  * Closes the communication socket, causing the server to remove the registered mailbox and to
  * delete possible non-read messages. The operation has no effects if the socket was not
  * connected.
  * @see m_iSocket  * @see IsConnected();
  */
void CRemoteMailbox::Disconnect()
{
 vprintmsg("Disconnecting from server");   VWAIT;
 close(m_iSocket);
 VOK;
};

/** Connects to server.
  * This function connects to the specificed host, trying to register a mailbox with the desired name.
  *
  * @param strMailboxName this is the name of the mailbox for your process you want to create.
  * @param strServerHostName this is the name of the server you want to connect to; this argument 
  *                           has the default value "localhost".
  * @param iPort  this is the server port number to which you want to connect; this argument has a
  *               default value of RMBX_SERVICE_PORT, which is originally set to 12345.
  * @return true if the connection has been successfully set up;
  *         false if it was not able to connect or there is another connected participant with 
  *               the desired name.
  * @see IsConnected();
  */
bool CRemoteMailbox::Connect( char * strMailboxName, 
			      char * strServerHostName, 
			      int iPort)
{
 struct hostent *   heHostInfo;
 char               rc;

 vprintmsg("Creating socket");  
 m_iSocket = socket(PF_INET,SOCK_STREAM,0);
 if (m_iSocket==-1) VFAIL(return false) else VOK;

 memset(&m_sinAddress,0,sizeof(m_sinAddress));
 m_sinAddress.sin_family      = AF_INET ;
 m_sinAddress.sin_port        = htons(RMBX_SERVICE_PORT);

 if (m_bVerbose) printf("Resolving host \"%s\"",strServerHostName); 
 vprintmsg("");  VWAIT;

 heHostInfo = gethostbyname(strServerHostName);
 if (!heHostInfo) VFAIL(return false) else VOK;
 
 if (m_bVerbose)
   printf("Resolved server address is %s\r\n",inet_ntoa(m_sinAddress.sin_addr));
 m_sinAddress.sin_addr = *(struct in_addr *) heHostInfo->h_addr;

 vprintmsg("Connecting to server");   VWAIT;
 if (connect(m_iSocket,(struct sockaddr *)&m_sinAddress,sizeof(m_sinAddress))==0) 
   VOK else VFAIL(return false);
 
 vprintmsg("Registering the writer's name");  VWAIT;
 if (send(m_iSocket,strMailboxName,strlen(strMailboxName)+1,0)<0)
  VFAIL(return false) else VOK;

 vprintmsg("Waiting for registration acknowledgement");  VWAIT;
 if (recv(m_iSocket,&rc,1,0)<0 || rc==0)
  VFAIL(return false) else VOK;

 m_bConnected=true;
}

/** Sets the verbosity level.
  * Use this function to override the default value of the verbosity.
  * When the verbosity level is true, the object sends results and
  * remarks to the process-associated terminal about the object's ongoing activities;
  * @param bVerbose set this value to true if you want to see results and
  * remarks on the object's ongoing activities; set it to false to disable them.
  * @see m_bVerbose
  */

void CRemoteMailbox::SetVerbosity(bool bVerbose)
{
  m_bVerbose = bVerbose;
}

/** Returns the connection status.
  * This method returns the current status of the communication.
  * 
  * @returns true if the mailbox-associated socket is connected to the mailboxd daemon.
  * @see Connect();
  */  
bool CRemoteMailbox::IsConnected()
{
  return m_bConnected;
}

/** Sends a message to a mailbox.
  * This function sends a message to the desired destination mailbox, waiting for an acknowledgment
  * return message from the server; due to this feature, the call could be blocking.
  * @return true if the message has been successfully delivered; false otherwise (this is a common 
  *  error when this mailbox client or the destination counterpart are not connected to the server, or the destination
  *  mailbox name was misspelled);
  * @param strDestination this is the name of the destination mailbox; this string is limited to RMBX_MAILBOX_ID_SIZE 
  *        characters in length.
  * @param strText this is the buffer containing the message that has to be delivered; this string is limited in length
  *        to the value of the RMBX_MAX_MESSAGE_SIZE constant. Since the lenght of this buffer is separately indicated,
  *        it does not need to be a zero-terminated string, but can contain any character. 
  * @param iLen contains the lenght of the message text buffer.
  * @see BroadcastMessage();
  * @see RMBX_MAX_MESSAGE_SIZE
  * @see RMBX_MAILBOX_ID_SIZE
*/
bool CRemoteMailbox::SendMessage(char * strDestination, char * strText, int iLen)
{
  if (!m_bConnected) return false;
  unsigned char chLen1 = iLen / 256,
                chLen2 = iLen % 256,
                chSuccess;
 
 vprintmsg("Sending message to receiver...");  VWAIT;
 
 if (write(m_iSocket,strDestination, strlen(strDestination)+1)<0) VFAIL(exit(EXIT_FAILURE));
 if (write(m_iSocket,&chLen1,1)<0)                                VFAIL(exit(EXIT_FAILURE));
 if (write(m_iSocket,&chLen2,1)<0)                                VFAIL(exit(EXIT_FAILURE));
 if (write(m_iSocket,strText, strlen(strText))<0)                 VFAIL(exit(EXIT_FAILURE)) else VOK;

 vprintmsg("Waiting for server's response");  VWAIT;
 
 if (recv(m_iSocket,&chSuccess,1,0)<0) VFAIL(exit(EXIT_FAILURE)) else VOK;
 if (chSuccess)
   { vprintmsg("Message correctly delivered to destination's mailbox"); VOK; return true;}
 else
   { vprintmsg("Message could not be delivered; unexisting destination ?"); VFAIL(return false);}
  
}

;
/** Sends a to all the registered mailbox.
  * This function sends a message to all the mailbox that have been registered onto the mailboxd daemon, 
  * waiting for an acknowledgment return message from the server; due to this feature, the call could be
  * blocking. Note that no destination mailbox parameter is requested.
  * @return true if the message has been successfully delivered; false otherwise (this is a common 
  *  error when this mailbox client or the destination counterpart is not connected to the server, or the destination
  *  mailbox name was misspelled;
  * @param strText this is the buffer containing the message that has to be delivered; this string is limited in length
  *        to the value of the RMBX_MAX_MESSAGE_SIZE constant. Since the lenght of this buffer is separately indicated,
  *        it does not need to be a zero-terminated string, but can contain any character. 
  * @param iLen contains the lenght of the message text buffer.
  * @see SendMessage();
  * @see RMBX_MAX_MESSAGE_SIZE
  */
bool CRemoteMailbox::BroadcastMessage(char * strText, int iLen)
{
 if (!m_bConnected) return false;
 
 unsigned char chLen1 = iLen / 256,
               chLen2 = iLen % 256,
               chSuccess;
 
 vprintmsg("Broadcasting message to receivers...");  VWAIT;
 
 if (write(m_iSocket,"\0",1)<0)                    VFAIL(exit(EXIT_FAILURE));
 if (write(m_iSocket,&chLen1,1)<0)                 VFAIL(exit(EXIT_FAILURE));
 if (write(m_iSocket,&chLen2,1)<0)                 VFAIL(exit(EXIT_FAILURE));
 if (write(m_iSocket,strText, strlen(strText))<0)  VFAIL(exit(EXIT_FAILURE)) else VOK;

 vprintmsg("Waiting for server's response");  VWAIT;
 
 if (recv(m_iSocket,&chSuccess,1,0)<0) VFAIL(exit(EXIT_FAILURE)) else VOK;
 if (chSuccess)
   { vprintmsg("Message correctly broadcasted to mailboxes."); VOK; return true;}
 else
   { vprintmsg("Message could not be broadcasted; connection problems ?"); VFAIL(return false);}
  
}

;
/** Receives a message.
  * This function receives the first message that has been enqueued in the mailbox registered onto
  * the mailboxd daemon by the calling process. 
  * If no messages are waiting, the call is blocking. 
  * @return true if the message has been successfully retrieved; 
  *         false otherwise (this is a common error when the supplied buffers are not large enough
  * to receive the data 
  * @param iLen contains the lenght of the message text buffer.
  * @see SendMessage();
  */

bool CRemoteMailbox::ReceiveMessage(char * strSender, int* pnchSender,
				    char * strMsgBody, int* pnchMsgBody)
{
  // a smarter control function could be implemented
  if (*pnchSender  < RMBX_MAILBOX_ID_SIZE)  return false;
  if (*pnchMsgBody < RMBX_MAX_MESSAGE_SIZE) return false;

  int iTotalRecChars=0,iRecChars=0, iMsgLen;
  char rc, chLen;

  vprintmsg("Waiting for messages");  VWAIT;
   while (true)
     {
       iRecChars = recv(m_iSocket,&rc,1,0);
       if (iRecChars<0) VFAIL(return false);
       //printf("%c",rc);
       strSender[iTotalRecChars++]=rc;
       if (!rc) break;
     }
   VOK;

   (*pnchSender)= iRecChars - 2;
   
   if (m_bVerbose)
     printf("Message sender: \"%s\"\n",strSender);
   
   vprintmsg("Extracting message length"); VWAIT;
   if (recv(m_iSocket,&chLen,1,MSG_WAITALL)<=0)
     {
       VFAIL_NE;
       Disconnect();
       m_bConnected=false;
       return false;
     }
   
   iMsgLen = chLen*256;
 
   VWAIT;
   if (recv(m_iSocket,&chLen,1,MSG_WAITALL)<=0) 
     {
       VFAIL_NE;
       Disconnect();
       m_bConnected=false;
       return false;
     }
  
   VOK;
   iMsgLen += chLen; 
  
   if (m_bVerbose)
     printf("Message is %i bytes in length\n",iMsgLen);
  
   vprintmsg("Extracting message text"); VWAIT;
   
   iRecChars = recv(m_iSocket,strMsgBody,iMsgLen,MSG_WAITALL);
   if (iRecChars<=0)
     {
       VFAIL_NE;
       Disconnect();
       m_bConnected=false;
       return false;
     }
     
   VOK;
   
   strMsgBody[iMsgLen]=0; 
   if (m_bVerbose)
     printf("Message text: \"%s\"\n",strMsgBody);
   
   (*pnchMsgBody) = iMsgLen;    
   return true;
}











