/************************************************************
*                                                           *
* Politecnico di Milano - University of Illinois at Chicago *
*                                                           *
*    Networked Operating Systems exam - Professor Piuri     *
*                                                           *
* Assignment by Daniele Paolo Scarpazza                     *
* Copyright (C) 2000, by Daniele Paolo Scarpazza.           *
*               All rights reserved.                        *
*                                                           *
************************************************************/

/*
  Note to code maintainers:

  WARNING: this program listing contains embedded documentation.
  Please respect writing conventions, otherwise modifications could
  result in inconsistent or partially missing generated documentation. 

  Daniele Paolo Scarpazza

*/

#ifndef CLASS_REMOTE_MAILBOX_H_INCLUDED
#define CLASS_REMOTE_MAILBOX_H_INCLUDED

/** Mailbox identifier maximum length. 
  * This is the maximum allowed length for mailbox identifier strings. This constant is simply used
  * to dimension internal buffers which receive the mailbox identifiers, and therefore can be simply 
  * set to a higher value without other modifications, no other limits are imposed.
  */ 
#define RMBX_MAILBOX_ID_SIZE        255

/** Message body maximum length.
  * This is the maximum allowed length for the message body (therefore not including source mailbox identifier, string terminator,
  * message length bytes).  This constant is currently used to dimension internal buffers which receive the mailbox identifiers, 
  * but it cannot be set to a higher value without other modifications, since currently two bytes are used to indicate the message
  * body length. 
  */  
#define RMBX_MAX_MESSAGE_SIZE     65535

/** Mailbox server port.
  * This is the standard port on which the mailboxd server is bound.
  */
#define RMBX_SERVICE_PORT         12345

/** Default verbosity level.
  * This is the default verbosity level for the object belonging to the CRemoteMailbox class.
  * It is always possible to override this setting by using the <I>SetVerbosity()</I> mathod.
  */ 
#define RMBX_DEFAULT_VERBOSITY     true

/** Remote mailbox client class.
  * This class represents a mailbox that is stored and managed by a remote host on the behalf of the process that owns the
  * mailbox itself and the object of this class which is associated to it.
  *  
  * This class exports a number of methods that can be used to connect as a client to a mailboxd daemon on a networked server, 
  * to register onto it with an unique identifier and create a mailbox on it, to write a message to another mailbox and
  * to read the mailbox.
  */ 

class CRemoteMailbox
{
 protected:
  /** Socket file descriptor number. 
    * This is the number of the file descriptor associated to the socket that is currently in use. 
    * This number is meaningless if the socket is not actually connected; this member variable is stored for internal use only,
    * its value is determined by the system at run-time.
    * @see m_bConnected
    * @see IsConnected
    */
  int                       m_iSocket;

  /** Connection flag.
    * This flag indicates whether the socket is connected or not. This member variable is stored for internal use only,
    * callers should read its value by using the <I>IsConnected()</I> method.
    * @see IsConnected
    */
  bool                      m_bConnected;

  /** Verbosity level.
    * This flag indicates whether the program should generate information and progress status messages during 
    * the program execution. Messages are sent to the associated terminal.
    * This flag is initially set to the default verbosity level; this member variable is stored for internal use only,
    * use instead the <I>SetVerbosity()</I> method to modify its value.  
    * @see SetVerbosity
    */
  bool                      m_bVerbose;

  /** Mailbox identifier.
    * This is the currently chosen mailbox identifier. This data member is meaningless is the mailbox is not
    * connected. This member variable is stored for internal use only, use instead the <I>Connect()</I> method
    * to set its value at connection time.  
    * @see IsConnected
    */
  char                      m_strMailBoxName[RMBX_MAILBOX_ID_SIZE+1];
  
  /** Server address.
    * This structure contains the inet address with host and port of the server on which the mailboxd server is running.
    * This member variable is stored for internal use only.
    */
  struct sockaddr_in        m_sinAddress;
 
 public:
  CRemoteMailbox();
  ~CRemoteMailbox();
  bool Connect( char * strMailboxName, 
		char * strServerHostName = "localhost", 
		int iPort = RMBX_SERVICE_PORT);
  void Disconnect();
  bool IsConnected();
  void SetVerbosity(bool bVerbose);
  bool SendMessage(char * strDestination, char * strText, int iLen);
  bool BroadcastMessage(char * strText, int iLen);
  bool ReceiveMessage(char * strSender,  int* pnchSender,
		      char * strMsgBody, int* pnchMsgBody);
};


#endif




















